import { stringify } from 'querystring';
import { BaseUtils } from '../';
import { NO_VALUE } from '../container_utils';
export abstract class TuyaDevice {
  protected deviceID: string;
  protected cloudResponseString: string;
  protected tuyaGetRequestGeneralData: TuyaGetRequestGeneralData;

  protected constructor(deviceID: string) {
    this.deviceID = deviceID;
    this.cloudResponseString = '';
    this.tuyaGetRequestGeneralData = new TuyaGetRequestGeneralData();
  }

  protected async retrieveGetRequestGeneralData(tuyaCloudResponse: string) {
    // Convert the cloud response to a string
    this.cloudResponseString = JSON.stringify(tuyaCloudResponse);

    // Extract whether the GET request passed or failed. In case of failure return.
    const succesMatch = this.cloudResponseString.match(/"\bsuccess\b":(.*?),/);
    const successTuya = succesMatch ? succesMatch[1] : null;

    if (successTuya == null || successTuya.toString() != 'true') {
      throw new Error('The GET request to Tuya Cloud failed!Exiting');
    }

    // Extract the unix timestamp of the request
    const timestampMatch = this.cloudResponseString.match(/"t":(\d+),/);
    const timestampTuya = timestampMatch ? timestampMatch[1] : null;
    if (timestampTuya == null) throw new Error('Could not retrieve unix timestamp!');

    // Extract the timezone
    const timezoneMatch = this.cloudResponseString.match(/"time_zone":"(.*?)"/);
    const timezoneTuya = timezoneMatch ? timezoneMatch[1] : null;
    if (timezoneTuya == null) throw new Error('Could not retrieve timezone!');

    // Refactor the time the request was sent at into a format that will be sent to ThingsBoard Cloud
    const thingsboardRequestTime = BaseUtils.unixTimestampToLocalTime13D(timestampTuya, timezoneTuya);
    if (thingsboardRequestTime == undefined)
      throw new Error('Could not convert the request time to designated format!');

    // Retrieve denice name
    const deviceNameMatch = this.cloudResponseString.match(/"name":"(.*?)",/);
    const deviceNameTuya = deviceNameMatch ? deviceNameMatch[1] : null;
    if (deviceNameTuya == null) throw new Error('Could not retrieve device name!');

    // Retrieve device status (online / offline)
    const deviceStatusMatch = this.cloudResponseString.match(/"online":(.*?),/);
    const deviceStatusTuya = deviceStatusMatch ? deviceStatusMatch[1] : null;
    if (deviceStatusTuya == null) throw new Error('Could not retrieve device status!');

    let thingsboardDeviceStatus = deviceStatusTuya == 'true' ? 'online' : 'offline';

    const updateTimeMatch = this.cloudResponseString.match(/"update_time":(\d+),/);
    const updateTime = updateTimeMatch ? updateTimeMatch[1] : null;
    if (updateTime == null) throw new Error('Could not retrieve when the device was last updated!');

    // Refactor the update time the request was sent at into a format that will be sent to ThingsBoard Cloud
    const thingsboardUpdateTime = BaseUtils.unixTimestampToLocalTime10D(updateTime, timezoneTuya);
    if (thingsboardUpdateTime == undefined) throw new Error('Could not convert the request time to designated format!');

    this.tuyaGetRequestGeneralData.setData(
      thingsboardRequestTime,
      successTuya.toString() == 'true',
      deviceNameTuya,
      thingsboardDeviceStatus,
      thingsboardUpdateTime,
    );
  }

  protected abstract retrieveGetRequestDeviceData(tuyaCloudResponse: string): void;

  protected abstract stringifyData(): string;
}

export class TuyaGetRequestGeneralData {
  private _timestamp: string;
  private _success: boolean;
  private _deviceName: string;
  private _deviceStatus: string; // this field indicates whether the device is online or not: true - device online, false - device offline
  private _updateTime: string;

  public constructor() {
    this._timestamp = '';
    this._success = false;
    this._deviceName = '';
    this._deviceStatus = '';
    this._updateTime = '';
  }

  // ======================= GETTERS & SETTERS =======================

  public setData(timestamp: string, success: boolean, deviceName: string, deviceStatus: string, updateTime: string) {
    this.timestamp = timestamp;
    this.success = success;
    this.deviceName = deviceName;
    this.deviceStatus = deviceStatus;
    this.updateTime = updateTime;
  }

  public get timestamp(): string {
    return this._timestamp;
  }

  public set timestamp(value: string) {
    this._timestamp = value;
  }

  public get success(): boolean {
    return this._success;
  }

  public set success(value: boolean) {
    this._success = value;
  }

  public get deviceName(): string {
    return this._deviceName;
  }

  public set deviceName(value: string) {
    this._deviceName = value;
  }

  public get deviceStatus(): string {
    return this._deviceStatus;
  }

  public set deviceStatus(value: string) {
    this._deviceStatus = value;
  }

  public get updateTime(): string {
    return this._updateTime;
  }

  public set updateTime(value: string) {
    this._updateTime = value;
  }
}
